from .utils import hex_to_int


class Trace:
    def __init__(self, dict):
        self.action = TraceAction(dict.get('action', {}))
        self.block_hash = dict.get('blockHash', dict.get('block_hash'))
        self.block_number = dict.get('blockNumber', dict.get('block_number'))
        self.result = TraceResult(dict.get('result', {}))
        self.subtraces = dict.get('subtraces')
        self.trace_address = dict.get(
            'traceAddress', dict.get('trace_address', []))
        self.transaction_hash = dict.get(
            'transactionHash', dict.get('transaction_hash'))
        self.transaction_position = dict.get(
            'transactionPosition', dict.get('transaction_position'))
        self.type = dict.get('type')
        self.error = dict.get('error')


class TraceAction:
    def __init__(self, dict):
        self.call_type = dict.get('callType', dict.get('call_type'))
        self.to = dict.get('to')
        self.input = dict.get('input')
        self.from_ = dict.get('from')
        self.value = hex_to_int(dict.get('value'))
        self.init = dict.get('init')
        self.address = dict.get('address')
        self.balance = dict.get('balance')
        self.refund_address = dict.get(
            'refundAddress', dict.get('refund_address'))


class TraceResult:
    def __init__(self, dict):
        self.gas_used = hex_to_int(dict.get('gasUsed', dict.get('gas_used')))
        self.address = dict.get('address')
        self.code = dict.get('code')



import sys
import os
from jsonc_parser.parser import JsoncParser
import sha3


def get_forta_config():
    config_flag_index = sys.argv.index(
        '--config') if '--config' in sys.argv else -1
    config_file = None if config_flag_index == - \
        1 else sys.argv[config_flag_index + 1]
    config_path = os.path.join(
        os.getcwd(), config_file if config_file else 'forta.config.json')
    return JsoncParser.parse_file(config_path)


def get_json_rpc_url():
    if 'JSON_RPC_HOST' in os.environ:
        return f'http://{os.environ["JSON_RPC_HOST"]}{":"+os.environ["JSON_RPC_PORT"] if "JSON_RPC_PORT" in os.environ else ""}'

    config = get_forta_config()
    if "jsonRpcUrl" not in config:
        raise Exception("no jsonRpcUrl found")
    return config["jsonRpcUrl"]


def create_block_event(dict):
    from .block_event import BlockEvent  # avoid circular import
    return BlockEvent(dict)


def create_transaction_event(dict):
    from .transaction_event import TransactionEvent  # avoid circular import
    return TransactionEvent(dict)


def assert_non_empty_string_in_dict(dict, key):
    assert_key_in_dict(dict, key)
    assert isinstance(dict[key], str) and len(
        dict[key]) > 0, f'{key} must be non-empty string'


def assert_enum_value_in_dict(dict, key, enum):
    assert_key_in_dict(dict, key)
    assert isinstance(dict[key], enum), f'{key} must be valid enum value'


def assert_key_in_dict(dict, key):
    assert key in dict, f'{key} is required'


def hex_to_int(strVal):
    return int(strVal, 16) if strVal and type(strVal) == str and strVal.startswith('0x') else int(strVal, 10)


def keccak256(val):
    hash = sha3.keccak_256()
    hash.update(bytes(val, encoding='utf-8'))
    return f'0x{hash.hexdigest()}'



import json
from enum import IntEnum
from .utils import assert_enum_value_in_dict, assert_non_empty_string_in_dict


class FindingSeverity(IntEnum):
    Unknown = 0
    Info = 1
    Low = 2
    Medium = 3
    High = 4
    Critical = 5


class FindingType(IntEnum):
    Unknown = 0
    Exploit = 1
    Suspicious = 2
    Degraded = 3


class Finding:
    def __init__(self, dict):
        assert_non_empty_string_in_dict(dict, 'name')
        assert_non_empty_string_in_dict(dict, 'description')
        assert_non_empty_string_in_dict(dict, 'alert_id')
        assert_enum_value_in_dict(dict, 'severity', FindingSeverity)
        assert_enum_value_in_dict(dict, 'type', FindingType)
        self.name = dict['name']
        self.description = dict['description']
        self.alert_id = dict['alert_id']
        self.protocol = dict.get('protocol', 'ethereum')
        self.severity = dict['severity']
        self.type = dict['type']
        self.everest_id = dict.get('everest_id')
        self.metadata = dict.get('metadata')

    def toJson(self):
        d = dict(self.__dict__, **
                 {'alertId': self.alert_id, 'everestId': self.everest_id})
        return json.dumps({k: v for k, v in d.items() if v or k == 'type' or k == 'severity'})



from .event_type import EventType
from .network import Network
from .transaction import Transaction
from .receipt import Receipt
from .trace import Trace
from .utils import keccak256


class TxEventBlock:
    def __init__(self, dict):
        self.hash = dict.get('hash')
        self.number = dict.get('number')
        self.timestamp = dict.get('timestamp')


class TransactionEvent:
    def __init__(self, dict):
        typeVal = dict.get('type', "BLOCK")
        self.type = EventType[typeVal] if type(
            typeVal) == str else EventType(typeVal)
        networkVal = dict.get('network', "MAINNET")
        self.network = Network[networkVal] if type(
            networkVal) == str else Network(networkVal)
        self.transaction = Transaction(dict.get('transaction', {}))
        self.receipt = Receipt(dict.get('receipt', {}))
        self.traces = list(map(lambda t: Trace(t), dict.get('traces', [])))
        self.addresses = dict.get('addresses', {})
        self.block = TxEventBlock(dict.get('block', {}))

    @property
    def hash(self):
        return self.transaction.hash

    @property
    def to(self):
        return self.transaction.to

    @property
    def from_(self):
        return self.transaction.from_

    @property
    def status(self):
        return self.receipt.status

    @property
    def gas_used(self):
        return self.receipt.gas_used

    @property
    def gas_price(self):
        return self.transaction.gas_price

    @property
    def logs(self):
        return self.receipt.logs

    @property
    def timestamp(self):
        return self.block.timestamp

    @property
    def block_number(self):
        return self.block.number

    @property
    def block_hash(self):
        return self.block.hash

    def filter_event(self, event_signature, contract_address=''):
        event_topic = keccak256(event_signature).lower()
        contract_address = contract_address.lower()
        events = filter(lambda log: len(log.topics) > 0 and
                        log.topics[0].lower() == event_topic and
                        (True if not contract_address else log.address.lower() == contract_address), self.receipt.logs)
        return list(events)
